@@ -221,7 +221,7 @@ gpgconf_read (void *engine, char *arg1, char *arg2,
 
   cfd[0].fd = rp[1];
 
-  status = _gpgme_io_spawn (gpgconf->file_name, argv, cfd, NULL);
+  status = _gpgme_io_spawn (gpgconf->file_name, argv, 0, cfd, NULL);
   if (status < 0)
     {
       _gpgme_io_close (rp[0]);
@@ -659,7 +659,7 @@ gpgconf_write (void *engine, char *arg1, char *arg2, gpgme_data_t conf)
 
   cfd[0].fd = rp[0];
 
-  status = _gpgme_io_spawn (gpgconf->file_name, argv, cfd, NULL);
+  status = _gpgme_io_spawn (gpgconf->file_name, argv, 0, cfd, NULL);
   if (status < 0)
     {
       _gpgme_io_close (rp[0]);
@@ -885,6 +885,7 @@ struct engine_ops _gpgme_engine_ops_gpgconf =
   {
     /* Static functions.  */
     _gpgme_get_gpgconf_path,
+    NULL,
     gpgconf_get_version,
     gpgconf_get_req_version,
     gpgconf_new,
@@ -911,6 +912,7 @@ struct engine_ops _gpgme_engine_ops_gpgconf =
     NULL,		/* trustlist */
     NULL,		/* verify */
     NULL,		/* getauditlog */
+    NULL,               /* opassuan_transact */
     gpgconf_conf_load,
     gpgconf_conf_save,
     gpgconf_set_io_cbs,
