@@ -48,10 +48,12 @@ _gpgme_op_data_lookup (gpgme_ctx_t ctx, ctx_op_data_id_t type, void **hook,
       data = calloc (1, sizeof (struct ctx_op_data) + size);
       if (!data)
 	return gpg_error_from_errno (errno);
+      data->magic = CTX_OP_DATA_MAGIC;
       data->next = ctx->op_data;
       data->type = type;
       data->cleanup = cleanup;
       data->hook = (void *) (((char *) data) + sizeof (struct ctx_op_data));
+      data->references = 1;
       ctx->op_data = data;
     }
   *hook = data->hook;
