@@ -397,7 +397,7 @@ build_commandline (char **argv)
 
 
 int
-_gpgme_io_spawn (const char *path, char **argv,
+_gpgme_io_spawn (const char *path, char * const argv[], unsigned int flags,
 		 struct spawn_fd_item_s *fd_list, pid_t *r_pid)
 {
   SECURITY_ATTRIBUTES sec_attr;
@@ -492,6 +492,9 @@ _gpgme_io_spawn (const char *path, char **argv,
 
   free (arg_string);
 
+  if (flags & IOSPAWN_FLAG_ALLOW_SET_FG)
+    _gpgme_allow_set_foreground_window ((pid_t)pi.dwProcessId);
+
   /* Insert the inherited handles.  */
   for (i = 0; fd_list[i].fd != -1; i++)
     {
@@ -533,8 +536,10 @@ _gpgme_io_spawn (const char *path, char **argv,
     int written;
     size_t len;
 
-    line[0] = '\n';
-    line[1] = '\0';
+    if ((flags & IOSPAWN_FLAG_ALLOW_SET_FG))
+      strcpy (line, "~1 \n");
+    else
+      strcpy (line, "\n");
     for (i = 0; fd_list[i].fd != -1; i++)
       {
 	/* Strip the newline.  */
@@ -674,3 +679,20 @@ _gpgme_io_dup (int fd)
     return fd;
 }
 
+
+extern "C"
+int
+_gpgme_io_socket (int domain, int type, int proto)
+{
+  errno = EIO;
+  return -1;
+}
+
+
+extern "C"
+int
+_gpgme_io_connect (int fd, struct sockaddr *addr, int addrlen)
+{
+  errno = EIO;
+  return -1;
+}
