@@ -1,21 +1,20 @@
 /* t-command.c - Regression test.
- * Copyright (C) 2009 g10 Code GmbH
- *
- * This file is part of GPGME.
- *
- * GPGME is free software; you can redistribute it and/or modify it
- * under the terms of the GNU Lesser General Public License as
- * published by the Free Software Foundation; either version 2.1 of
- * the License, or (at your option) any later version.
- *
- * GPGME is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, see <https://gnu.org/licenses/>.
- * SPDX-License-Identifier: LGPL-2.1-or-later
+   Copyright (C) 2009 g10 Code GmbH
+
+   This file is part of GPGME.
+
+   GPGME is free software; you can redistribute it and/or modify it
+   under the terms of the GNU Lesser General Public License as
+   published by the Free Software Foundation; either version 2.1 of
+   the License, or (at your option) any later version.
+   
+   GPGME is distributed in the hope that it will be useful, but
+   WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+   Lesser General Public License for more details.
+   
+   You should have received a copy of the GNU Lesser General Public
+   License along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -35,41 +34,36 @@
       if (err)							\
         {							\
           fprintf (stderr, "%s:%d: %s: %s (%d.%d)\n",        	\
-                   __FILE__, __LINE__, gpgme_strsource (err),	\
-		   gpgme_strerror (err),                        \
-                   gpgme_err_source (err), gpgme_err_code (err)); \
+                   __FILE__, __LINE__, gpg_strsource (err),	\
+		   gpg_strerror (err),                          \
+                   gpg_err_source (err), gpg_err_code (err));	\
           exit (1);						\
         }							\
     }								\
   while (0)
 
 
-static gpgme_error_t
+static gpg_error_t
 data_cb (void *opaque, const void *data, size_t datalen)
 {
-  (void)opaque;
-  (void)data;
-
   printf ("DATA_CB: datalen=%d\n", (int)datalen);
   return 0;
-}
+}     
 
 
-static gpgme_error_t
+static gpg_error_t
 inq_cb (void *opaque, const char *name, const char *args,
         gpgme_data_t *r_data)
 {
   gpgme_data_t data;
-  gpgme_error_t err;
-
-  (void)opaque;
+  gpg_error_t err;
 
   if (name)
     {
       printf ("INQ_CB: name=`%s' args=`%s'\n", name, args);
       /* There shall be no data object.  */
       assert (!*r_data);
-
+      
       err = gpgme_data_new (&data);
       fail_if_err (err);
       *r_data = data;
@@ -87,29 +81,26 @@ inq_cb (void *opaque, const char *name, const char *args,
   /* Uncomment the next lines and send a "SCD LEARN" to test sending
      cancel from in inquiry.  */
   /* if (name && !strcmp (name, "KNOWNCARDP")) */
-  /*   return gpgme_error (GPG_ERR_ASS_CANCELED); */
+  /*   return gpg_error (GPG_ERR_ASS_CANCELED); */
 
 
   return 0;
-}
+}     
 
 
-static gpgme_error_t
+static gpg_error_t
 status_cb (void *opaque, const char *status, const char *args)
 {
-  (void)opaque;
-
   printf ("STATUS_CB: status=`%s'  args=`%s'\n", status, args);
   return 0;
-}
+}     
 
 
 
-int
+int 
 main (int argc, char **argv)
 {
   gpgme_error_t err;
-  gpgme_error_t op_err;
   gpgme_ctx_t ctx;
   const char *command;
 
@@ -126,7 +117,7 @@ main (int argc, char **argv)
       argv++;
     }
   command = argc? *argv : "NOP";
-
+  
 
   err = gpgme_new (&ctx);
   fail_if_err (err);
@@ -134,9 +125,18 @@ main (int argc, char **argv)
   err = gpgme_set_protocol (ctx, GPGME_PROTOCOL_ASSUAN);
   fail_if_err (err);
 
-  err = gpgme_op_assuan_transact_ext (ctx, command, data_cb, NULL,
-                                  inq_cb, NULL, status_cb, NULL, &op_err);
-  fail_if_err (err || op_err);
+  err = gpgme_op_assuan_transact (ctx, command,
+                                  data_cb, NULL,
+                                  inq_cb, NULL,
+                                  status_cb, NULL);
+  fail_if_err (err);
+  err = gpgme_op_assuan_result (ctx)->err;
+  if (err)
+    fprintf (stderr, "assuan command `%s' failed: %s <%s> (%d)\n", 
+             command, gpg_strerror (err), gpg_strsource (err), err);
+  else
+    fprintf (stderr, "assuan command `%s' succeeded\n", command);
+
 
   gpgme_release (ctx);
 
